(function($) {

  document.addEventListener('DOMContentLoaded', function() {
    const debug = false;
    const isAssignation = !!document.getElementById('corona_test_results_assign');
    const isRegistration = !!document.getElementById('corona_test_results_register');
    const isIE11 = !!window.MSInputMethodContext && !!document.documentMode;

    // dismissable notices
    jQuery(function($) {
      $( document ).on( 'click', '[data-dismiss-id] .notice-dismiss', function () {
          const id = $( this ).closest( '[data-dismiss-id]' ).data( 'dismiss-id' );
          $.ajax( ajaxurl,
            {
              type: 'POST',
              data: {
                action: 'corona_test_results_dismiss_notice',
                id: id,
              }
          } );
      } );
    });

    function isConnected(node) {
      if (!('isConnected' in Node.prototype)) {
        return !node.ownerDocument ||
        !(
          node.ownerDocument.compareDocumentPosition(node) &
          node.DOCUMENT_POSITION_DISCONNECTED
        )
      }

      return node.isConnected
    }

    const downloadBlob = function(blob, fileName) {
      if ('msSaveOrOpenBlob' in window.navigator) {
        window.navigator.msSaveOrOpenBlob(blob, fileName);
        return;
      }

      const blobUrl = URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = blobUrl;
      link.download = fileName;
      document.body.append(link);
      link.click();
      link.remove();
      window.setTimeout(function() {
        URL.revokeObjectURL(link.href)
      }, 1000 );
    };

    switch(true) {
      /**
       * Test result assignation
       */
      case isAssignation:
        const assignationForm = document.getElementById('corona-test-results-form');
        const searchField = document.getElementById('corona_test_results_search');
        const paginationContainer = document.querySelector('.tablenav-pages');
        const exportButton = document.getElementById('corona_test_results_export');
        const resultsTable = document.getElementById('corona_test_results_table');
        const resultsTableBody = resultsTable.querySelector('tbody');
        const dirtyStates = [];
        let isFormDirty = false;
        let isFormSubmitting = false;

        searchField.disabled = false;
        exportButton.disabled = false;

        let resultsTableRows = [].slice.call(resultsTableBody.querySelectorAll('tr'));

        const registerDirtyHandler = function() {
          let selects = resultsTableRows.map(function(row) { return row.querySelector('select'); });

          const makeDirty = function(s) {
            isFormDirty = true;
            dirtyStates.push(s);
          };

          window.addEventListener('beforeunload', function(ev) {
            if (isFormDirty && !isFormSubmitting) {
              ev.preventDefault();
              ev.returnValue = '';
              return '';
            }

            delete ev['returnValue'];
            return undefined;
          });

          selects.forEach(function(s) {
            s && s.addEventListener('change', function() { makeDirty(s) });
          });
        }

        const searchFilter = function() {
          const searchString = searchField.value.toUpperCase();

          if (paginationContainer) {
            paginationContainer.style.display = searchString.length ? 'none' : '';
            if (!searchString.length && typeof updatePageItems !== 'undefined') {
              updatePageItems();
              return;
            }
          }

          if (resultsTableRows.length) {
            resultsTableRows.forEach(function(r) {
              let firstColumn = r.querySelector('td');
              let rowCode = firstColumn.textContent.toUpperCase();
              let isMatching = rowCode.indexOf(searchString) === 0;

              if (isMatching) {
                resultsTableBody.appendChild(r);
                r.style.display = '';
              } else if (r.parentNode === resultsTableBody) {
                resultsTableBody.removeChild(r)
              }
            });
          }
        };

        // pagination
        let updatePageItems;
        if (paginationContainer) {
          const btnFirst = paginationContainer.querySelector('.button.first-page');
          const btnPrev = paginationContainer.querySelector('.button.prev-page');
          const btnNext = paginationContainer.querySelector('.button.next-page');
          const btnLast = paginationContainer.querySelector('.button.last-page');

          const itemsPerPage = parseInt(paginationContainer.querySelector('#pagination-items-per-page').value, 10);
          const totalPages = parseInt(paginationContainer.querySelector('.total-pages').textContent.trim(), 10);
          const currentPageInput = paginationContainer.querySelector('#current-page-selector');
          const pagedFormInput = assignationForm.querySelector('[name="paged"]');
          let currentPage = parseInt(currentPageInput.value, 10);

          const updatePagination = function() {
            btnFirst.disabled = currentPage <= 2;
            btnPrev.disabled = currentPage === 1;
            btnNext.disabled = currentPage === totalPages;
            btnLast.disabled = currentPage >= totalPages - 1;
            currentPageInput.value = pagedFormInput.value = currentPage;
          };

          btnFirst.addEventListener('click', function() {
            currentPage = 1;
            updatePageItems();
          });

          btnPrev.addEventListener('click', function() {
            currentPage--;
            updatePageItems();
          });

          btnNext.addEventListener('click', function() {
            currentPage++;
            updatePageItems();
          });

          btnLast.addEventListener('click', function() {
            currentPage = totalPages;
            updatePageItems();
          });

          updatePageItems = function() {
            const itemsFrom = (currentPage - 1) * itemsPerPage;
            const itemsTo = itemsFrom + itemsPerPage - 1;

            resultsTableRows.forEach(function(item, index) {
              const isMatching = index >= itemsFrom && index <= itemsTo;
              if (isMatching) {
                resultsTableBody.appendChild(item);
                item.style.display = '';
              } else if (item.parentNode === resultsTableBody) {
                resultsTableBody.removeChild(item)
              }
            });
            updatePagination();
          };

          currentPageInput.addEventListener('input', function() {
            let newPage = parseInt(this.value);
            if (isNaN(newPage) || newPage < 1 || newPage > totalPages) {
              return;
            }
            currentPage = newPage;
            updatePageItems();
          });
          currentPageInput.addEventListener('blur', function() {
            this.value = currentPage;
          });

          updatePageItems();
          updatePagination();
          // fetch remaining rows via ajax
          if (resultsTableRows.length && 'ctr_fetch_hidden_rows' in window) {
            let selectTemplate = resultsTableRows[0].querySelector('select').cloneNode(true);

            const loadingRow = document.createElement('tr');
            loadingRow.innerHTML = '<td colspan="3">' + corona_test_results.rows_loading + '</td>';
            resultsTableBody.appendChild(loadingRow);            

            $.ajax( ajaxurl,
              {
                type: 'POST',
                data: {
                  action: 'corona_test_results_fetch_rows',
                  stati: window.ctr_fetch_hidden_rows,
                },
                success: function (data) {
                  if ( typeof data !== 'undefined' && Array.isArray(data) ) {
                    data.forEach(function (row) {
                      selectTemplate.querySelector('[selected]').removeAttribute('selected');
                      selectTemplate.querySelector('option[value="' + row.s + '"]').setAttribute('selected', true);
                      selectTemplate.name = 'corona_test_results_result_status[' + row.c + ']';
                      const tr = document.createElement('tr');
                      tr.innerHTML = '<td><code>' + row.c +  '</code></td><td>' + selectTemplate.outerHTML + '</td><td data-datetime="' + row.t + '">' + row.d + '</td>';
                      resultsTableRows.push(tr);
                    });
                    resultsTableBody.removeChild(loadingRow);
                    updatePageItems();
                    updatePagination();
                    registerDirtyHandler();
                    searchFilter();
                  } else {
                    console.error(corona_test_results.rows_loading_failed);
                  }
                },
                error: function() {
                  console.error(corona_test_results.rows_loading_failed);
                  loadingRow.firstChild.innerHTML = '<span style="color: red;">' + corona_test_results.rows_loading_failed + '</span>';
                }
            } );
          }
        }

        registerDirtyHandler();

        searchField.addEventListener('input', searchFilter);

        exportButton.addEventListener('click', function() {
          alert(corona_test_results.premiumFeatureNotice);
        });

        assignationForm.addEventListener('submit', function(ev) {
          isFormSubmitting = true;

          const hiddenStateChanges = {};

          if (dirtyStates.length) {
            dirtyStates.forEach(function(select) {
              if(!isConnected(select)) {
                hiddenStateChanges[select.parentNode.parentNode.firstElementChild.innerText.trim()] = select.options[select.selectedIndex].value;
              }
            });

            const hiddenInput = document.createElement('input');
            hiddenInput.type = 'hidden';
            hiddenInput.name = 'corona_test_results_result_status_hidden';
            hiddenInput.value = JSON.stringify(hiddenStateChanges);
            assignationForm.appendChild(hiddenInput);
          }
        });
        break;
      /**
       * Code and PDF generation
       */
      case isRegistration:
        const form = document.getElementById('corona-test-results-form');
        const submitBtn = document.getElementById('submit');
        const privacyHint = document.getElementById('privacy-hint');
        const popupHint = document.getElementById('popup-hint');
        const generateBtn = document.getElementById('generate-pdf');
        const resetBtn = document.getElementById('reset-form');
        const codeField = document.getElementById('test_result_code');
        const message = document.getElementById('message');

        function disableButtons() {
          submitBtn.disabled = true;
          generateBtn.disabled = true;
        }

        function enableButtons() {
          submitBtn.disabled = false;
          generateBtn.disabled = false;
        }

        function switchButtons() {
          // switch submit and generate button, show reset button
          submitBtn.style.display = 'none';
          privacyHint.style.display = 'none';
          popupHint.style.display = 'block';
          generateBtn.style.display = '';
          resetBtn.style.display = '';
        }

        function resetForm() {
          let inputs = form.querySelectorAll('input');
          // for IE11...
          inputs = [].slice.call(inputs);

          inputs.forEach(function(i) { i.value = '' });
          submitBtn.style.display = '';
          privacyHint.style.display = 'block';
          popupHint.style.display = 'none';
          generateBtn.style.display = 'none';
          resetBtn.style.display = 'none';
        }

        function handleAjaxErrors(response, isBatch) {
          if (typeof isBatch === 'undefined') {
            isBatch = false;
          }

          // coult be const, but for IE11 array workaround...
          let paragraphs = message.querySelectorAll('p');
          let activeParagraph;
          if (response.responseJSON && response.responseJSON.data && response.responseJSON.data.error) {
            // error during code generation
            activeParagraph = 'codegen' + (isBatch ? '-batch' : '');
          } else {
            // something else went wrong
            activeParagraph = 'ajax';
          }
          // for IE11...
          paragraphs = [].slice.call(paragraphs);
          paragraphs.forEach(function (p) {
            p.style.display = p.classList.contains('vt-error-' + activeParagraph) ? 'block' : 'none';
          });
          message.style.display = 'block';
          submitBtn.disabled = false;
          batchBtn.disabled = false;
        }

        pdfMake.fonts = {
          Roboto: {
            normal: 'Roboto-Regular.ttf',
            bold: 'Roboto-Medium.ttf',
            italics: 'Roboto-Italic.ttf',
            bolditalics: 'Roboto-MediumItalic.ttf'
          },
          RobotoMono: {
              normal: 'RobotoMono-Regular.ttf'
          },
          RobotoBold: {
              normal: 'Roboto-Bold.ttf'
          }
        };

        /**
         * pdfmake extensions
         */
        function mapTableBodies(innerTableCell) {
          const findInlineHeight = this.findInlineHeight(
            innerTableCell,
            maxWidth,
            usedWidth
          );

          usedWidth = findInlineHeight.width;
          return findInlineHeight.height;
        }

        function findInlineHeight(cell, maxWidth, usedWidth) {
          usedWidth = typeof usedWidth === 'undefined' ? 0 : usedWidth;
          let calcLines = function(inlines) {
            if (!inlines)
              return {
                height: 0,
                width: 0,
              };
            let currentMaxHeight = 0;
            let lastHadLineEnd = false;
            inlines.forEach(function(currentNode) {
              usedWidth += currentNode.width;
              if (usedWidth > maxWidth || lastHadLineEnd) {
                currentMaxHeight += currentNode.height;
                usedWidth = currentNode.width;
              } else {
                currentMaxHeight = Math.max(currentNode.height, currentMaxHeight);
              }
              lastHadLineEnd = !!currentNode.lineEnd;
            });

            return {
              height: currentMaxHeight,
              width: usedWidth,
            };
          }
          if (cell._offsets) {
            usedWidth += cell._offsets.total;
          }
          if (cell._inlines && cell._inlines.length) {
            return calcLines(cell._inlines);
          }  else if (cell.stack && cell.stack[0]) {
            return cell.stack.map(function(item) {
              return findInlineHeight(item, maxWidth);
            }).reduce(function(prev, next) {
              return {
              height: prev.height + next.height,
              width: Math.max(prev.width + next.width)
              };
            });
          } else if (cell.table) {
            let currentMaxHeight = 0;
            cell.table.body.forEach(function(currentTableBodies) {
              const innerTableHeights = currentTableBodies.map(mapTableBodies);
              innerTableHeights.push(currentMaxHeight);
              currentMaxHeight =  Math.max.apply(this, innerTableHeights);
            })
            return {
              height: currentMaxHeight,
              width: usedWidth,
            };
          } else if (cell._height) {
            usedWidth += cell._width;
            return {
              height: cell._height,
              width: usedWidth,
            };
          }

          return {
            height: null,
            width: usedWidth,
          };
        }

        function applyVerticalAlignment(node, rowIndex, align) {
            const allCellHeights = node.table.body[rowIndex].map(
              function(innerNode, columnIndex) {
                const mFindInlineHeight = findInlineHeight(
                  innerNode,
                  node.table.widths[columnIndex]._calcWidth
                );
                return mFindInlineHeight.height;
              }
            );
            let maxRowHeight = Math.max.apply(this, allCellHeights);

            node.table.body[rowIndex].forEach(function(cell, ci) {
              if (allCellHeights[ci] && maxRowHeight > allCellHeights[ci]) {
                let topMargin;

                let cellAlign = align;
                if (Array.isArray(align)) {
                    cellAlign = align[ci];
                }

                if (cellAlign === 'bottom') {
                  topMargin = maxRowHeight - allCellHeights[ci];
                } else if (cellAlign === 'center') {
                  topMargin = (maxRowHeight - allCellHeights[ci]) / 2;
                }

                if (topMargin) {
                    if (cell._margin) {
                      cell._margin[1] = topMargin;
                    } else {
                      cell._margin = [0, topMargin, 0, 0];
                    }
                }
              }
            });
        }
        /**
         * EOF pdfmake extensions
         */

        function prepareTemplateString(stringIn, isSingle) {
          let stringOut = stringIn;

          if (typeof isSingle === 'undefined') {
            isSingle = false;
          }

          if (!stringOut.length) return '';

          // hard-replace {{testdate}}
          stringOut = stringOut.replace( /\{\{testdate\}\}/g, corona_test_results.documentTexts.date_i18n, stringOut );

          // if no custom field is used, but the custom field placeholder is present, remove it
          if ( typeof corona_test_results.cfg.template_customfield === 'undefined' || !corona_test_results.cfg.template_customfield.length ) {
            // if it would cause a line break, remove that as well
            stringOut = stringOut.replace( /((^|\r?\n)\s*\{\{customfield\}\}\s*(\r?\n|$))/g, function( matches, match1, match2, contents, offset, input_string ) {
              if ( /\n/.test(match1) && /\n/.test(match2) ) {
                return "\n";
              }

              return '';
            });
            // replace occurrences mid-string
            stringOut = stringOut.replace( /\{\{customfield\}\}/g, "");
          }

          stringOut = stringOut.replace( /\{\{([a-z_]+)\}\}/gi, function(matches, contents) {
            if (!isSingle) {
              return '';
            }

            let matchingInput = form.querySelector('input[name="' + contents + '"]');
            let fieldValue = matchingInput ? matchingInput.value : null;

            if (fieldValue && matchingInput.type.toLowerCase() === 'date' && typeof window.Intl !== 'undefined' && typeof window.Intl.DateTimeFormat !== 'undefined') {
              const locale = document.documentElement.getAttribute('lang');
              try {
                fieldValue = new Intl.DateTimeFormat(locale, {day: 'numeric', month: 'long', year: 'numeric'}).format(matchingInput.valueAsDate);
              } catch(e) {}
            }
            return fieldValue ? fieldValue : '';
          });

          return parseTextMarkup(stringOut.trim());
        }

        function parseTextMarkup(stringIn) {
          let textOut = [];

          const fragment = document.createDocumentFragment();
          const root = document.createElement('div');
          fragment.appendChild(root);
          root.innerHTML = stringIn;

          const childNodes = [].slice.call(root.childNodes);
          childNodes.forEach(function(node) {
            recursiveMarkupParser(node, textOut, {});
          });
          return textOut;
        }

        function recursiveMarkupParser(node, textStack, props) {
          if (node.nodeType === 3) {
            textStack.push(Object.assign({}, { text: node.textContent }, props));
          } else {
            props.bold = props.bold || (node.tagName === 'B' || node.tagName === 'STRONG');
            props.italics = props.italics || (node.tagName === 'I' || node.tagName === 'EM');

            const childNodes = [].slice.call(node.childNodes);
            childNodes.forEach(function(node) {
              recursiveMarkupParser(node, textStack, Object.assign({}, { text: node.textContent }, props));
            });
          }
        }

        function getPdfDocTemplate(codes) {
          const dd = {
            pageSize: 'A5',
            pageMargins: [ 30, 30, 30, 30 ],
            content: [],
            styles: {
              headline: {
                fontSize: 12,
                bold: true
              }
            },
            defaultStyle: {
              fontSize: 10,
              lineHeight: 1.2,
              bold: false
            }
          };

          if (typeof corona_test_results.cfg.template_poweredby !== 'undefined' && !!corona_test_results.cfg.template_poweredby && corona_test_results.cfg.template_poweredby !== 'off') {
            dd.background = function(currentPage, pageSize) {
              return {
                fontSize: 7.5,
                alignment: 'center',
                absolutePosition: { x: 0, y: 550 },
                text: parseTextMarkup(corona_test_results.documentTexts.powered_by)
              }
            };
          }

          if (corona_test_results.cfg.template_logoimage) {
            dd.images = {
              logo: corona_test_results.cfg.template_logoimage
            }
          }

          const isSingle = codes.length === 1;

          const borderNone = [ false, false, false, false ];
          const borderBottom = [ false, false, false, true ];

          const surname = prepareTemplateString('{{surname}}', isSingle);
          const firstname = prepareTemplateString('{{firstname}}', isSingle);
          const dateofbirth = prepareTemplateString('{{dateofbirth}}', isSingle);

          const patientDataTable = [];

          if (isSingle && typeof corona_test_results.cfg.template_customfield !== 'undefined' && corona_test_results.cfg.template_customfield.length) {
            const customfield = prepareTemplateString('{{customfield}}', isSingle);
            patientDataTable.push([ {font: 'RobotoBold', text: corona_test_results.cfg.template_customfield + ':', border: borderNone, margin: [ 0, 0, 10, 0 ] }, { text: customfield, border: customfield.length ? borderNone : borderBottom } ]);
          }

          patientDataTable.push([ {font: 'RobotoBold', text: corona_test_results.documentTexts.labels.surname + ':', border: borderNone, margin: [ 0, 0, 10, 0 ] }, { text: surname, border: surname.length ? borderNone : borderBottom } ]);
          patientDataTable.push([ {font: 'RobotoBold', text: corona_test_results.documentTexts.labels.firstname + ':', border: borderNone, margin: [ 0, 0, 10, 0 ] }, { text: firstname, border: firstname.length ? borderNone : borderBottom } ]);
          patientDataTable.push([ {font: 'RobotoBold', text: corona_test_results.documentTexts.labels.dateofbirth + ':', border: borderNone, margin: [ 0, 0, 10, 0 ] }, { text: dateofbirth, border: dateofbirth.length ? borderNone : borderBottom } ]);
          patientDataTable.push([ {font: 'RobotoBold', text: corona_test_results.documentTexts.labels.testdate + ':', border: borderNone, margin: [ 0, 0, 10, 0 ] }, { text: corona_test_results.documentTexts.date_i18n, border: borderNone } ]);

          const tb_topleft = prepareTemplateString(corona_test_results.cfg.template_tb_topleft, isSingle);
          const tb_topright = prepareTemplateString(corona_test_results.cfg.template_tb_topright, isSingle);
          const tb_before = prepareTemplateString(corona_test_results.cfg.template_tb_before, isSingle);
          const tb_after = prepareTemplateString(corona_test_results.cfg.template_tb_after, isSingle);
          const tb_bottom = prepareTemplateString(corona_test_results.cfg.template_tb_bottom, isSingle);

          codes.forEach(function(code, index) {
            const logoImage = {};
            if (corona_test_results.cfg.template_logoimage && corona_test_results.cfg.template_logoimage.length) {
              logoImage.image = 'logo';
              logoImage.width = 120;
              logoImage.margin = [0, 0, 0, 5];
            }

            if (index > 0) {
              dd.content.push({
                pageBreak: 'before',
                text: null
              });
            }

            dd.content.push({
              table: {
                widths: ['*', 120],
                body: [
                  [
                    tb_topleft,
                    logoImage
                  ],
                ]
              },
              layout: {
                paddingTop: function (index, node) {
                    applyVerticalAlignment(node, index, ['bottom', 'top']);
                    return 0;
                },
                hLineWidth: function(i) {
                  return 0;
                },
                vLineWidth: function(i) {
                  return 0;
                },
                paddingLeft: function(i) {
                  return i && 4 || 0;
                },
                paddingRight: function(i, node) {
                  return (i < node.table.widths.length - 1) ? 4 : 0;
                }
              }
            },
            {
              table: {
                widths: ['*', 120],
                body: [
                  [ { style: 'headline', text: [ '\n', corona_test_results.cfg.template_tb_salutation ] }, JSON.parse(JSON.stringify(tb_topright)) ],
                ]
              },
              layout: {
                paddingTop: function (index, node) {
                  applyVerticalAlignment(node, index, ['bottom', 'bottom']);
                  return 0;
                },
                hLineWidth: function(i) {
                  return 0;
                },
                vLineWidth: function(i) {
                  return 0;
                },
                paddingLeft: function(i) {
                  return i && 4 || 0;
                },
                paddingRight: function(i, node) {
                  return (i < node.table.widths.length - 1) ? 4 : 0;
                }
              }
            },
            {
              margin: [0, 20, 0, 0],
              text: tb_before
            },
            {
                text: '\n' + corona_test_results.resultsBaseUrl + '\n\n',
                bold: true,
                alignment: 'center'
            },
            {
              text: tb_after
            },
            {
              text: '\n'
            },
            {
              fontSize: 20,
              alignment: 'center',
              text: [
                {
                  bold: true,
                  text: 'Code: '
                },
                {
                  font: 'RobotoMono',
                  characterSpacing: 3,
                  text: code
                }
              ]
            },
            '\n',
            {
              qr: corona_test_results.resultsBaseUrl + '#' + code,
              alignment: 'center',
              fit: 100
            },
            '\n\n',
            {
              text: tb_bottom,
            },
            {
              pageBreak: 'before',
              text: null
            },
            {
              table: {
                widths: ['*', 120],
                body: [
                  [
                    '',
                    logoImage
                  ],
                ]
              },
              layout: {
                paddingTop: function (index, node) {
                  applyVerticalAlignment(node, index, ['bottom', 'top']);
                  return 0;
                },
                hLineWidth: function(i) {
                  return 0;
                },
                vLineWidth: function(i) {
                  return 0;
                },
                paddingLeft: function(i) {
                  return i && 4 || 0;
                },
                paddingRight: function(i, node) {
                  return (i < node.table.widths.length - 1) ? 4 : 0;
                }
              }
            },
            {
              table: {
                widths: ['*', 120],
                body: [
                  [ { style: 'headline', text: corona_test_results.documentTexts.onsite_notice }, JSON.parse(JSON.stringify(tb_topright))],
                ]
              },
              layout: {
                paddingTop: function (index, node) {
                  applyVerticalAlignment(node, index, ['bottom', 'bottom']);
                  return 0;
                },
                hLineWidth: function(i) {
                  return 0;
                },
                vLineWidth: function(i) {
                  return 0;
                },
                paddingLeft: function(i) {
                  return i && 4 || 0;
                },
                paddingRight: function(i, node) {
                  return (i < node.table.widths.length - 1) ? 4 : 0;
                }
              }
            },
            {
              margin: [ 0, 35 ],
              fontSize: 16,
              lineHeight: 1,
              table: {
                widths: ['auto', '*'],
                body: JSON.parse(JSON.stringify(patientDataTable))
              },
              layout: {
                hLineColor: 'gray',
                paddingTop: function (index, node) {
                  applyVerticalAlignment(node, index, ['center', 'center']);
                  return 5;
                },
                hLineWidth: function(i, node) {
                  return 1;
                },
                vLineWidth: function(i) {
                  return 0;
                },
                paddingLeft: function(i) {
                  return i && 5 || 0;
                },
                paddingRight: function(i, node) {
                  return (i < node.table.widths.length - 1) ? 5 : 0;
                }
              }
            },
            {
              fontSize: 20,
              alignment: 'center',
              text: [
                {
                  bold: true,
                  text: 'Code: '
                },
                {
                  font: 'RobotoMono',
                  characterSpacing: 3,
                  text: code
                }
              ]
            });
          });

          return dd;
        }

        function generatePDF(codes) {
          disableButtons();
          if (!codeField.value && typeof codes === 'undefined') {
            console.error('Cannot generate PDF: Code has not been generated yet.');
            return;
          }

          if (typeof codes === 'undefined') {
            codes = [codeField.value];
          }

          var docDefinition = getPdfDocTemplate(codes);

          const callback = function() {
            enableButtons();
            if (codes.length > 1) {
              batchExportPdfBtn.disabled = false;
            } else {
              switchButtons();
            }
          };

          if (isIE11 || codes.length > 1) {
            pdfMake.createPdf(docDefinition).download((codes.length === 1 ? codeField.value : 'corona-test-results_Batch') + '.pdf').then(callback);
          } else {
            const pdfWindow = window.open('');
            if (!pdfWindow) {
              // if the window has been blocked by a popup blocker, try to fall back to downloading
              pdfMake.createPdf(docDefinition).download((codes.length === 1 ? codeField.value : 'corona-test-results_Batch') + '.pdf').then(callback);
            } else {
              pdfMake.createPdf(docDefinition).open(pdfWindow).then(callback);
            }
          }
        }

        resetForm();
        if (debug === true) {
          surname.value = 'Mustermann';
          firstname.value = 'Maximilian';
          dateofbirth.value = '1970-01-01';
        }

        submitBtn.addEventListener('click', function() {
          disableButtons();
          // add and get a new code via Ajax
          if (!codeField.value) {
            const data = {
              action: 'corona_test_results_ajax_getcode'
            };

            $.ajax({
              type : "post",
              dataType : "json",
              url: ajaxurl,
              data: data,
              success: function(response) {
                message.style.display = 'none';
                codeField.value = response.test_result_code;
                generatePDF();
              },
              error: function(response) {
                handleAjaxErrors(response);
              }
            });
          } else {
            generatePDF();
          }
        });

        generateBtn.addEventListener('click', function() { generatePDF() });
        resetBtn.addEventListener('click', function() { resetForm() });

        const batchBtn = document.getElementById('generate-batch');
        const batchExportCsvBtn = document.getElementById('batch-export-csv');
        const batchExportPdfBtn = document.getElementById('batch-export-pdf');
        const batchActions = document.getElementById('batch-actions');
        const batchCountInput = document.getElementById('batch-count');
        let lastBatch = {
          codes: []
        };

        batchBtn.addEventListener('click', function() {
          alert(corona_test_results.premiumFeatureNotice);
        });

        break;
    }
  });
})(jQuery);
