<?php
( defined( 'ABSPATH' ) && is_admin() && is_user_logged_in() && current_user_can( 'manage_options' ) ) or die;

/**
 * register page
 */
function corona_test_results_register_settings() {
	$license_error = get_transient( 'corona_test_results_license_error' );

	add_submenu_page(
		'corona-test-results',
		vt_helper__default_i18n__( 'Settings'),
		vt_helper__default_i18n__( 'Settings' ) . ( $license_error ? " <span class='update-plugins count-1'><span class='update-count'>!</span></span>" : '' ),
		'manage_options',
		'corona-test-results-settings',
		'corona_test_results_settings_page'
	);
}
add_action( 'admin_menu', 'corona_test_results_register_settings' );

/**
 * check for setting errors and display error messages
 */
function corona_test_results_settings_check_errors() {
	$options = corona_test_results_get_options();

	$license_error = get_transient( 'corona_test_results_license_error' );
	$license_success = get_transient( 'corona_test_results_license_activated' );

	if ( $license_error ) {
		add_settings_error(
			'corona-test-results-settings',
			'corona-test-results-settings',
			$license_error,
			'error'
		);
		delete_transient( 'corona_test_results_license_error' );
	} else if ( !empty( $options['license_key'] ) ) {
		add_settings_error(
			'corona-test-results-settings',
			'corona-test-results-settings',
			__( 'Your license key was activated successfully!', 'corona-test-results' )
			. ' '
			. sprintf(
				// translators: %s: Link to the plugin upload page
				__( 'Visit the %s page in order to upgrade the plugin, uploading the file provided to you after purchase.', 'corona-test-results' ),
				'<a href="' . esc_attr( esc_url( corona_test_results_get_add_plugins_url() ) ) . '">' . vt_helper__default_i18n__( 'Add Plugins' ) . '</a>'
			),
			'success'
		);
	}/*%remove_light%*/ else if ( $license_success) {
		add_settings_error(
			'corona-test-results-settings',
			'corona-test-results-settings',
			$license_success,
			'success'
		);
	}/*%/remove_light%*/

	$result_pages = array( 'result_retrieval' );

	$duplicates = array();

	foreach ( $result_pages as $page) {
		$pp = corona_test_results_get_page( $page, true );
		$pid = $pp && $pp->post_status !== 'trash' ? $pp->ID : 0;

		if ( empty( $pid ) ) {
			$message = null;

			if ( $pp && $pp->post_status === 'trash' ) {
				// translators: %s: result page name
				$message = __( 'The page for &quot;%s&quot; has been moved to trash.', 'corona-test-results' );
			} else if ( !empty( $options[ 'page_' . $page ] ) ) {
				// translators: %s: result page name
				$message = __( 'The page for &quot;%s&quot; does no longer exist.', 'corona-test-results' );
			} else {
				if( !empty( $options['license_key'] ) || $page === 'result_retrieval' ) {
					// translators: %s: result page name
					$message = __( 'The page for &quot;%s&quot; is not set.', 'corona-test-results' );
				}
			}

			if ( isset($message) && !empty( $message ) ) {
				add_settings_error(
					'corona-test-results-settings',
					'corona-test-results-settings',
					sprintf(
						$message,
						corona_test_results_get_page_name( $page )
					),
					'error'
				);
			}

		} else foreach ( $result_pages as $page_compare) {
			if ( $page === $page_compare ) continue;

			if ( $pid === corona_test_results_get_page_id( $page_compare )
				&& !empty( $pid )
				&& !in_array( $page_compare, $duplicates )
			) {
				$duplicates[] = $page;

				add_settings_error(
					'corona-test-results-settings',
					'corona-test-results-settings',
					// translators: %s: the two result page names that are set to the same page
					sprintf(
						__( '&quot;%s&quot; is set to the same page as &quot;%s&quot;.', 'corona-test-results' ),
						corona_test_results_get_page_name( $page_compare ),
						corona_test_results_get_page_name( $page )
					),
					'error'
				);
			}
		}
	}
}

/**
 * register setting page sections and options
 */
function corona_test_results_register_options() {
    global $corona_test_results_cfg;

    register_setting( 'corona_test_results_options', 'corona_test_results_options', array(
        'type' => 'array',
        'sanitize_callback' => 'corona_test_results_options_validate',
        'default' => $corona_test_results_cfg
    ));

	$pageslug = 'corona_test_results_settings';
	$premiumLabel = '<span class="premium-label">Premium</span>';

	$premiumlabelTag = ' ' . $premiumLabel;

	$section = 'vt_code_generation';
    add_settings_section( $section, esc_html__('Code Generation', 'corona-test-results') . $premiumlabelTag, 'corona_test_results_opts_code_description', $pageslug );
    add_settings_field( 'corona_test_results_opts_code_length', __('Code Length', 'corona-test-results'), 'corona_test_results_opts_code_length', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_code_blocklist', __('Additional block list', 'corona-test-results'), 'corona_test_results_opts_code_blocklist', $pageslug, $section );

	$section = 'vt_pdf_template';
	add_settings_section( $section, esc_html__('PDF Template', 'corona-test-results') . $premiumlabelTag, 'corona_test_results_opts_template_description', $pageslug );

    add_settings_field( 'corona_test_results_opts_template_customfield', __('Custom Field', 'corona-test-results'), 'corona_test_results_opts_template_customfield', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_topleft', __('Text block top left', 'corona-test-results'), 'corona_test_results_opts_template_tb_topleft', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_logoimage', __('Logo Image', 'corona-test-results'), 'corona_test_results_opts_template_logoimage', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_topright', __('Text block top right', 'corona-test-results'), 'corona_test_results_opts_template_tb_topright', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_salutation', __('Text block salutation', 'corona-test-results'), 'corona_test_results_opts_template_tb_salutation', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_before', __('Text block before URL', 'corona-test-results'), 'corona_test_results_opts_template_tb_before', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_after', __('Text block after URL', 'corona-test-results'), 'corona_test_results_opts_template_tb_after', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_tb_bottom', __('Text block bottom', 'corona-test-results'), 'corona_test_results_opts_template_tb_bottom', $pageslug, $section );
    add_settings_field( 'corona_test_results_opts_template_poweredby', __('Show &quot;powered by&quot; Notice', 'corona-test-results'), 'corona_test_results_opts_template_poweredby', $pageslug, $section );

	$section = 'vt_assignation';
	add_settings_section( $section, esc_html__('Assignation', 'corona-test-results') . $premiumlabelTag, null, $pageslug );
    add_settings_field( 'corona_test_results_opts_assignation_entries_per_page', __('Entries per page', 'corona-test-results'), 'corona_test_results_opts_assignation_entries_per_page', $pageslug, $section );

	$section = 'vt_result_pages';
	add_settings_section( $section, esc_html__('Result Page & Content', 'corona-test-results'), 'corona_test_results_opts_pages_description', $pageslug );
	$result_pages = corona_test_results_get_result_pages_slugs();
	foreach ($result_pages as $result_page_slug) {
		$result_page_premiumlabel = ( $result_page_slug === 'result_retrieval' ) ? '' : ( '<br>' . $premiumlabelTag );
		add_settings_field( 'corona_test_results_opts_pages_' . $result_page_slug, corona_test_results_get_page_name( $result_page_slug ) . $result_page_premiumlabel, 'corona_test_results_opts_pages_' . $result_page_slug, $pageslug, $section );
	}

	$section = 'vt_license';
	// translators: %s: formatted string "Premium"
	add_settings_section( $section, sprintf( esc_html__('%s License Activation', 'corona-test-results'), $premiumLabel ), 'corona_test_results_opts_license_description', $pageslug );
	add_settings_field( 'corona_test_results_opts_license_key', __('License Key', 'corona-test-results'), 'corona_test_results_opts_license_key', $pageslug, $section );
}
add_action( 'admin_init', 'corona_test_results_register_options' );

function corona_test_results_settings_enqueue_media( $page ) {
	if ( $page === corona_test_results_admin_page_slug() ) {
		wp_enqueue_media();
		// wp_register_script('media-uploader', plugins_url('media-uploader.js' , __FILE__ ), array('jquery'));
		// wp_enqueue_script('jquery');
	}
}
add_action('admin_enqueue_scripts', 'corona_test_results_settings_enqueue_media');

/**
 * code generation options
 */
function corona_test_results_opts_code_description() {
    echo '<p>' . sprintf(esc_html__(
        'The codes for assignation and retrieval of test results are generated automatically using uppercase alphanumeric characters, with the exception of the easily confused letters and numbers %s.', 'corona-test-results'),
        '<code>O</code>, <code>0</code>, <code>I</code>, <code>1</code>'
        ) .
        '<br>' . esc_html__(
        'Some easy-to-guess combinations and an internal list of bad words will also be avoided, which further reduces the number of available combinations.', 'corona-test-results'
        ) . '</p>';
}

function corona_test_results_opts_code_length() {
    $options = corona_test_results_get_options();

	$option_name = '';

    echo "<input id='corona_test_results_opts_code_length' name='$option_name' type='number' value='" . esc_attr( $options['code_length'] ) . "' min='" . CORONA_TEST_RESULTS_MIN_CODE_LENGTH . "' max='" . CORONA_TEST_RESULTS_CODE_COLUMN_LENGTH . "' class='small-text' />";
}

function corona_test_results_opts_code_blocklist() {
    $options = corona_test_results_get_options();

	$option_name = '';

    echo '<p>'. esc_html__('If a generated code would include one of the strings entered in the field below (separated by comma, space or line break), that code will be discarded and a new one will be generated instead.', 'corona-test-results') . '<br>';
    echo '<strong>' . esc_html__('Note:', 'corona-test-results'). '</strong> ' . esc_html__('Blocking too many short strings or single characters can greatly reduce the number of available combinations or even result in failing code generation.', 'corona-test-results') . '</p>';
    echo "<textarea class='large-text code' id='corona_test_results_opts_code_blocklist' name='$option_name' rows='5' cols='50'>" . esc_html( $options['code_blocklist'] ) . "</textarea>";
}

/**
 * PDF template options
 */
function corona_test_results_opts_template_description() {
	$placeholders = array(
		'surname',
		'firstname',
		'dateofbirth',
		'customfield',
		'testdate'
	);

    echo '<p>' . esc_html__(
        'Change the text blocks and logo appearing on the generated PDF documents.', 'corona-test-results'
        ) . '<br>';
    echo esc_html__(
        'Available placeholders in text blocks:', 'corona-test-results'
        ) . ' <code>{{' . implode( '}}</code>, <code>{{', $placeholders ) . '}}</code></p>';
}

function corona_test_results_opts_template_customfield() {
    $options = corona_test_results_get_options();

	$option_name = '';

	$value = isset( $options['template_customfield'] ) ? $options['template_customfield'] : '';

	echo "<p><input id='corona_test_results_opts_template_customfield' name='$option_name' type='text' value='" . esc_attr( $value ) . "' class='regular-text'></p>";
	echo "<p class='description'>" . esc_html__( "Leave empty if you don't need an additional field, e.g. for a patient ID. Otherwise, this will be the label for the field in the test registration form.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_tb_topleft() {
    global $corona_test_results_cfg_defaults;
	$options = corona_test_results_get_options();

	$option_name = '';

	echo "<textarea rows='4' class='regular-text' id='corona_test_results_opts_template_tb_topleft'
			name='$option_name'>"
			. esc_html( $options['template_tb_topleft'] ) . "</textarea>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_topleft'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' )
			. "</button>";
	echo "<p class='description'>" . esc_html__( "Usually used to display the test subject's data as entered in the test registration form.", 'corona-test-results' ) . ' ' . esc_html__( "Displayed only on the handout for the tested person.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_logoimage() {
    $options = corona_test_results_get_options();

	$option_name = '';

	$currentUrl = isset( $options['template_logoimage'] ) ? $options['template_logoimage'] : '';

	?>
	<img id="corona_test_results_opts_template_logoimage_preview" style="cursor: pointer; display: none;" onerror="this.style.display = 'none';" onload="this.style.display = 'block'; this.style.maxWidth = document.querySelector('.regular-text').offsetWidth + 'px'" alt="" />
	<input id="corona_test_results_opts_template_logoimage" type="text" name="<?php echo $option_name ?>" value="<?php echo esc_attr( $currentUrl ) ?>" class="regular-text" />
	<input id="corona_test_results_opts_template_logoimage_picker" type="button" class="button-primary" value="<?php esc_attr_e( 'Select Image', 'corona-test-results' ) ?>" />
	<script>
	document.addEventListener('DOMContentLoaded', function() {
		var $ = jQuery;

		var mediaUploader;
		var $mediaInput = $('#corona_test_results_opts_template_logoimage');
		var $mediaButton = $('#corona_test_results_opts_template_logoimage_picker, #corona_test_results_opts_template_logoimage_preview');
		var $mediaPreview = $('#corona_test_results_opts_template_logoimage_preview');
		var mediaCurrentUrl = '<?php echo esc_attr( $currentUrl ) ?>';

		if (mediaCurrentUrl.length) {
			$mediaPreview.attr( 'src', mediaCurrentUrl );
		}

		$($mediaButton).add($mediaPreview).click(function(e) {
		  e.preventDefault();
			if (mediaUploader) {
			mediaUploader.open();
			return;
		  }
		  mediaUploader = wp.media.frames.file_frame = wp.media({
			title: 'Choose Image',
			button: {
			text: 'Choose Image'
		  }, multiple: false });
		  mediaUploader.on('select', function() {
			var attachment = mediaUploader.state().get('selection').first().toJSON();
			$mediaInput.val(attachment.url);
			$mediaPreview.attr('src', attachment.url);
		  });
		  mediaUploader.open();
		});

		$($mediaInput).on('input', function (e) {
			$mediaPreview.attr('src', e.target.value);
		});
	  });
	  </script>
	  <?php
}

function corona_test_results_opts_template_tb_topright() {
    global $corona_test_results_cfg_defaults;
    $options = corona_test_results_get_options();

	$option_name = '';

	echo "<textarea rows='4' class='regular-text' id='corona_test_results_opts_template_tb_topright'
			name='$option_name'>"
			. esc_html( vt_helper__retranslate_option( 'template_tb_topright' ) ) . "</textarea>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_topright'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' )
			. "</button>";
	echo "<p class='description'>" . esc_html__( "Usually used to provide information about the testing location, e.g. a medical practice's name and contact data.", 'corona-test-results' ) . ' ' . esc_html__( "Displayed on both the handout and the on-site record.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_tb_salutation() {
    global $corona_test_results_cfg_defaults;
    $options = corona_test_results_get_options();

	$option_name = '';

	echo "<p><input id='corona_test_results_opts_template_tb_salutation'
			name='$option_name'
			type='text' value='" . esc_attr( vt_helper__retranslate_option( 'template_tb_salutation' ) )
			. "' class='regular-text'>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_salutation'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' ) . "</button>";
	echo "<p class='description'>" . esc_html__( "Displayed only on the handout for the tested person.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_tb_before() {
    global $corona_test_results_cfg_defaults;
    $options = corona_test_results_get_options();

	$option_name = '';

	echo "<textarea rows='4' class='regular-text' id='corona_test_results_opts_template_tb_before'
			name='$option_name'>"
			. esc_html( vt_helper__retranslate_option( 'template_tb_before' ) ) . "</textarea>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_before'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' )
			. "</button>";
	echo "<p class='description'>" . esc_html__( "Displayed only on the handout for the tested person.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_tb_after() {
    global $corona_test_results_cfg_defaults;
    $options = corona_test_results_get_options();

	$option_name = '';

	echo "<textarea rows='4' class='regular-text' id='corona_test_results_opts_template_tb_after'
			name='$option_name'>"
			. esc_html( vt_helper__retranslate_option( 'template_tb_after' ) ) . "</textarea>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_after'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' )
			. "</button>";
	echo "<p class='description'>" . esc_html__( "Displayed only on the handout for the tested person.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_tb_bottom() {
    global $corona_test_results_cfg_defaults;
    $options = corona_test_results_get_options();

	$option_name = '';

	echo "<textarea rows='4' class='regular-text' id='corona_test_results_opts_template_tb_bottom'
			name='$option_name'>"
			. esc_html( vt_helper__retranslate_option( 'template_tb_bottom' ) ) . "</textarea>
			<button type='button'
				onclick='this.previousElementSibling.value = \""
				. preg_replace( '/\n/', '\\\n', esc_attr__( $corona_test_results_cfg_defaults['template_tb_bottom'], 'corona-test-results' ) )
				. "\";' class='button'>" . esc_html__( 'Reset To Default', 'corona-test-results' )
			. "</button>";
	echo "<p class='description'>" . esc_html__( "Usually used for a data privacy statement.", 'corona-test-results' ) . ' ' . esc_html__( "Displayed only on the handout for the tested person.", 'corona-test-results' ) . "</p>";
}

function corona_test_results_opts_template_poweredby() {
	$options = corona_test_results_get_options();
    $options = corona_test_results_get_options();

	$option_name = '';

	$checked = ( isset( $options['template_poweredby'] ) && $options['template_poweredby'] === 'on' ) ? ' checked' : '';
	echo "<p><input type='checkbox' id='corona_test_results_opts_template_poweredby' name='$option_name' value='on'$checked /> <label for='corona_test_results_opts_template_poweredby'>" . esc_html__('Support plugin developer', 'corona-test-results') . "</label></p>";
}

/**
 * assignation page options
 */
function corona_test_results_opts_assignation_entries_per_page() {
    $options = corona_test_results_get_options();

	$option_name = '';

    echo "<input id='corona_test_results_opts_code_length' name='$option_name' type='number' value='" . esc_attr( $options['assignation_entries_per_page'] ) . "' min='1' class='small-text' />";
}

/**
 * result page and content pages
 */
function corona_test_results_opts_pages_description() {
    echo '<p>' . esc_html__(
        'The result page should contain a form for checking the test result for a given code. Its URL is displayed on the generated PDF document.', 'corona-test-results'
        ) . '<br>';
    echo esc_html__(
        'The content pages only provide the contents to be displayed depending on the test status and therefore do not have to be published. It is recomended to leave them in draft status, so they are not publicly accessible without entering a valid code.', 'corona-test-results'
        ) . '</p>';
}

function corona_test_results_opts_pages_result__helper( $page_slug ) {
    $options = corona_test_results_get_options();

	$option_name = ( $selected_page = ( $page_slug === 'result_retrieval' ) ? 'corona_test_results_options[page_' . $page_slug . ']' : '');
	$selected_page = ( $page_slug === 'result_retrieval' ) ? corona_test_results_get_page_id( $page_slug ) : 0;

	$pages_selection = wp_dropdown_pages(
		array(
			'name'              => $option_name,
			'class'				=> 'vt-select-page',
			'show_option_none'  => vt_helper__default_i18n__( '&mdash; Select &mdash;' ),
			'option_none_value' => '0',
			'selected'          => $selected_page,
			'post_status'       => array( 'draft', 'publish' ),
		)
	);
	if ( !$pages_selection ) {
		echo '<p><select disabled><option>' . esc_html__( 'No pages available' ) . '</option></p>';
	} else {
		echo "<span style='" . (empty( $selected_page ) ? 'display:none;' : '') . "'><a href='" . esc_attr( get_site_url() . '?page_id=' . ($selected_page ? $selected_page : 0) . '&preview=true&force_result_status=' . corona_test_results_get_page_state( $page_slug, true ) ) . "' target='_blank'>" . esc_html( vt_helper__default_i18n__( 'Preview' ) ) . "</a>";
		echo " | <a href='" . esc_attr( get_admin_url() . 'post.php?post=' . ($selected_page ? $selected_page : 0) . '&action=edit' ) . "' target='_blank'>" . esc_html( vt_helper__default_i18n__( 'Edit' ) ) . "</a></span>";
	}

	if ( 'result_retrieval' === $page_slug ) {
		echo '<p class="description">'
			. sprintf(
				// translators: %s: self-closing shortcode for the retrieval form
				esc_html__( 'Use the shortcode %s to embed the result retrieval form.', 'corona-test-results' ),
				'<code>[testresults_form]</code>'
			)
			. '<br>'
			. sprintf(
				// translators: %s: encapsulating shortcode for the retrieval form
				esc_html__( 'If you want to use a custom-styled input and button, put your markup inside %s, making sure to set the following attributes on the <input> element:', 'corona-test-results' ),
				'<code>[testresults_form open]&hellip;[testresults_form close]</code>'
			)
			. '<br><code>name="testresult_code" id="testresult_code"</code>'
			. ' '
			. sprintf(
				// translators: %s: (1) noerrors attribute (2) shortcode for form errors
				esc_html__( 'To influence where error messages appear, set the %s attribute on the form shortcode and use the shortcode %s where you want the error messages to appear.', 'corona-test-results' ),
				'<code>noerrors</code>',
				'<code>[testresults_errors]</code>'
			)
			. '</p>';
	} else {
		echo '<p class="description">';
		echo sprintf(
			// translators: %s: (1,2) formatted shortcodes
			esc_html__( 'Use the shortcode %s to display the code that was entered, and %s to display a pre-formatted, color-coded box with the result.', 'corona-test-results' ),
			'<code>[testresults_code]</code>',
			'<code>[testresults_code formatted]</code>'
		);
		echo '</p>';
	}
}

function corona_test_results_opts_pages_result_retrieval() {
	corona_test_results_opts_pages_result__helper( 'result_retrieval' );
}

function corona_test_results_opts_pages_result_pending() {
	corona_test_results_opts_pages_result__helper( 'result_pending' );
}

function corona_test_results_opts_pages_result_positive() {
	corona_test_results_opts_pages_result__helper( 'result_positive' );
}

function corona_test_results_opts_pages_result_negative() {
	corona_test_results_opts_pages_result__helper( 'result_negative' );
}

/**
 * license
 */
function corona_test_results_opts_license_description() {
    echo '<p>' . sprintf(
		esc_html__(
		// translators: %s: (1) opening link tag, (2) formatted string "Premium", (3) closing link tag
        'After you %sbuy the %s version%s, enter your license key here to register it to this domain.', 'corona-test-results'
        ),
		'<a href="' . corona_test_results_premium_shop_url() . '" target="_blank">',
		'<strong><em>Premium</em></strong>',
		'</a>'
	)
	 . '</p>';
}

function corona_test_results_opts_license_key() {
    $options = corona_test_results_get_options();

	$option_name = empty( $options['license_key'] ) ? 'corona_test_results_options[license_key]' : '';

	$readonly = $option_name ? '' : ' readonly';

	echo "<p><input id='corona_test_results_opts_license_key' name='$option_name' type='text' value='" . esc_attr( $options['license_key'] ) . "' class='regular-text'$readonly></p>";
	echo "<p class='description'><strong>" . esc_html__('Note:', 'corona-test-results')
		. "</strong> " . esc_html__( 'The license will be bound to the current domain of this WordPress installation. Once activated, it will only be possible to activate it on this domain again in the future and cannot be transferred to a different domain.', 'corona-test-results' )
		. "<br><em>"
		.	sprintf(
			// translators: Plugin author name linking to the plugin author URI
			esc_html__( 'By trying to activate a license, you agree to share the entered license key and the current domain of this WordPress installation with %s.', 'corona-test-results' ),
			corona_test_results_get_author_link()
		)
		. "</em></p>";
}

/**
 * settings validation
 */
function corona_test_results_options_validate( $input ) {
	global $corona_test_results_cfg;

	$options = corona_test_results_get_options();

    $output = $input;

	// code length min / max (do not reset when the value is not even sent, e.g. after license deactivation)
	if ( isset( $input['code_length'] ) ) {
		if ((int)$input['code_length'] < CORONA_TEST_RESULTS_MIN_CODE_LENGTH) {
			$output['code_length'] = CORONA_TEST_RESULTS_MIN_CODE_LENGTH;
		} else if ((int)$input['code_length'] > CORONA_TEST_RESULTS_CODE_COLUMN_LENGTH) {
			$output['code_length'] = CORONA_TEST_RESULTS_CODE_COLUMN_LENGTH;
		} else {
			$output['code_length'] = (int)$input['code_length'];
		}
	}

	// checkbox handling
	$checkbox_options = ['template_poweredby'];
	foreach ($checkbox_options as $option_name) {
		$output[$option_name] = ( isset($input[$option_name]) && $input[$option_name] === 'on') ? 'on' : 'off';
	}

	if ( !empty( $input['license_key'] ) && ( !isset( $options['license_key'] ) || $options['license_key'] !== $input['license_key'] )) {
		$wp_response = wp_remote_post( 'https://shop.48design.com/vierachtdesign_licensing.php', array(
			'body' => array(
				'key' => trim($input['license_key']),
				'pid' => 'b05bd1e415c8eedd93ed10302f88be8457f73aaa23db1ebd13e06f155b063903', // data needed by the licensing validation process
				'uuid' => $_SERVER[ 'HTTP_HOST' ]
			)
		));

		$general_error = __( 'An error occurred while trying to verify your license key. Please try again in a short while.', 'corona-test-results' );
		$invalid_key_error = __( 'The key that you entered is invalid.', 'corona-test-results' );
		$different_uuid = __( 'This license is already registered to a different domain.', 'corona-test-results' );
		$not_verified = nl2br( sprintf(
			// translators: support link with text "contact us"
			__( "The activation request for the current domain could not be verified. This will usually happen on development servers that are not publically available.\nIf this is happening to you on a live domain, please %s to have this domain whitelisted!", 'corona-test-results' ),
			corona_test_results_get_support_mail_link( __( 'contact us', 'corona-test-results' ) )
		));
		$result_error = null;

		if ( is_wp_error( $wp_response ) ) {
			$result_error = $wp_response->get_error_message();
		} else if ( empty($wp_response['body']) ) {
			$result_error = $general_error;
		} else {
			$response = json_decode( $wp_response['body'], true );
			if ( !isset( $response['success'] ) || true !== $response['success'] ) {
				$status = isset( $response['data'] ) && isset( $response['data']['status'] ) ? $response['data']['status'] : null;
				$status_code = isset( $response['code'] ) ? $response['code'] : null;
				switch ($status_code) {
					case 'license_already_activated_for_different_uuid':
						$result_error = $different_uuid;
						break;
					case 'request_server_not_verified':
						$result_error = $not_verified;
						break;
					default:
						switch ($status) {
							case 404:
								$result_error = $invalid_key_error;
								break;
							default:
								$result_error = $general_error;
						}
				}
			}
		}

		if ( !empty( $result_error ) ) {
			set_transient( 'corona_test_results_license_error', $result_error, 10 );
			unset( $output['license_key'] );
		}
	}

	// merge with existing settings object, so that options not existing in this form for whatever reason will not be deleted
	$output = array_merge($corona_test_results_cfg, $output);

    return $output;
}

/**
 * settings page contents
 */
function corona_test_results_settings_page() {
	$options = corona_test_results_get_options();
    ?>
	<div class="wrap">
    <h1><?php esc_html_e('Test Results Settings', 'corona-test-results'); ?></h1>
	<?php
	corona_test_results_check_review_nag();

	if ( empty( $options['license_key'] ) ) {
		$premium_string = '<strong><em>Premium</em></strong>';
		$upsell_message = nl2br(
			esc_html__( "You are using the free version of the plugin.", 'corona-test-results')
			. ' ' . sprintf(
				// translators: %s: formatted string "Premium"
				esc_html__( "All %s settings will not be saved, and result pages will always display their default content.", 'corona-test-results' ),
				$premium_string
				)
			. '<br>' . sprintf(
				// translators: %s: (1) formatted string "Premium", (2,3) opening and closing link tags
				esc_html__( "%sGet the %s version%s in order to customize code generation, documents and result page contents.", 'corona-test-results' ),
				'<a href="' . esc_url( corona_test_results_premium_shop_url() ) . '" target="_blank">',
				$premium_string,
				'</a>'
			)
		);
		echo "<div class='notice notice-warning'><p>$upsell_message</p></div>";
	}

	corona_test_results_settings_check_errors();
	settings_errors( 'corona-test-results-settings' );
	?>
	<style>
		.premium-label {
			color: #aa4800;
			display: inline-block;
			font-style: italic;
			font-weight: 600;
		}
	</style>
    <form action="options.php" method="post">
        <?php
        settings_fields( 'corona_test_results_options' );
        do_settings_sections( 'corona_test_results_settings' ); ?>
        <input name="submit" class="button button-primary" type="submit" value="<?php echo esc_attr( vt_helper__default_i18n__( 'Save Changes' ) ); ?>" />
    </form>
	</div>
    <?php
}

/**
 * Add a post display state for special WC pages in the page list table.
 *
 * @param array   $post_states An array of post display states.
 * @param WP_Post $post        The current post object.
 */
function corona_test_results_add_post_states( $post_states, $post ) {
	$result_pages = corona_test_results_get_result_pages_slugs();

	foreach ( $result_pages as $page_slug ) {
		if ( corona_test_results_get_page_id( $page_slug ) === $post->ID ) {
			if ( 'result_retrieval' !== $page_slug ) {
				unset($post_states['draft']);
			}
			$post_states['vt_page_for' . $page_slug] = corona_test_results_get_page_name( $page_slug );
		}
	}

    return $post_states;
}
add_filter( 'display_post_states', 'corona_test_results_add_post_states', 10, 2 );
