<?php
defined( 'ABSPATH' ) or die;

global $corona_test_results_cfg;

/**
 * default config
 * IMPORTANT: The current language is not yet available here,
 * but we still use the translation functions for the translator tool to pick them up.
 * The default values have to be used as parameters for translation functions again
 * in order to have them translated to the actually set language.
 */
$corona_test_results_cfg = $corona_test_results_cfg_defaults = array(
	'license_key' => '',
    'code_length' => CORONA_TEST_RESULTS_MIN_CODE_LENGTH, // maxmimum defined in constant CORONA_TEST_RESULTS_CODE_COLUMN_LENGTH (or adapt database VARCHAR column and update constant!)
    'code_blocklist' => '',
	'assignation_entries_per_page' => 15,
	'template_papersize' => 'a5',
	'template_tb_topright' => "created with\n<strong>Corona Test Results\nfor WordPress</strong>",
	'template_tb_topleft' => "{{customfield}}\n{{surname}}\n{{firstname}}\n{{dateofbirth}}",
	'template_tb_salutation' => __( 'Dear patient,', 'corona-test-results' ),
	'template_tb_before' => __( 'the result for your test taken on {{testdate}} will be ready within 24 hours to up to 3 days and can then be retrieved at', 'corona-test-results' ),
	'template_tb_after' => __( 'using the following code. Alternatively, you can scan the QR code with your mobile device to get directly to your result.', 'corona-test-results' ),
	'template_tb_bottom' => __( "Data Privacy Notice:\nThe security of your data is important to us. Therefore, it is not possible to draw any conclusions about your person using the online result check.", 'corona-test-results' ),
	'template_poweredby' => 'on'
);

$saved_options = get_option( 'corona_test_results_options' );
if (is_array($saved_options)) {

	$saved_options = array_filter( $saved_options, function($k) {
		return 'page_' === substr( $k, 0, 5 ) || in_array( $k, array('license_key') );
	}, ARRAY_FILTER_USE_KEY);

	$corona_test_results_cfg = array_merge($corona_test_results_cfg_defaults, $saved_options);
} else {
	$corona_test_results_cfg = $corona_test_results_cfg_defaults;
}

$corona_test_results_page_cache = array();

/**
 * get current configuration
 */
function corona_test_results_get_options() {
	global $corona_test_results_cfg;
    return $corona_test_results_cfg;
}

/**
 * result page (ID, permalink) retrieval
 */
function corona_test_results_get_page( $pagename, $include_trashed = false ) {
	global $corona_test_results_cfg, $corona_test_results_page_cache;

	$found_page = null;
	$returnValue = null;

	if ( isset( $corona_test_results_page_cache[$pagename] ) ) {
		$found_page = $corona_test_results_page_cache[$pagename];
	} else if (
		isset( $corona_test_results_cfg['page_' . $pagename] )
		&& !empty( $corona_test_results_cfg['page_' . $pagename] )
	) {
		$corona_test_results_page_cache[$pagename] = $found_page = get_post( $corona_test_results_cfg['page_' . $pagename] );
	}

	if ($found_page && ( $include_trashed || $found_page->post_status !== 'trash' ) ) {
		$returnValue = $found_page;
	}

	return $returnValue;
}

function corona_test_results_get_page_id( $pagename = null) {
	$page = corona_test_results_get_page( $pagename );
	return $page ? $page->ID : null;
}

function corona_test_results_get_page_url( $pagename = 'result_retrieval' ) {
	$page = corona_test_results_get_page( $pagename );
	return $page ? get_permalink($page) : null;
}

function corona_test_results_get_page_state( $page_slug, $as_int = false ) {
	$states = array(
		'result_retrieval' => null,
		'result_pending' => $as_int ? 0 : __( 'Pending', 'corona-test-results' ),
		'result_positive' => $as_int ? 1 : __( 'Positive', 'corona-test-results' ),
		'result_negative' => $as_int ? 2 : __( 'Negative', 'corona-test-results' ),
	);

	if ( isset($states[$page_slug]) ) {
		return $states[$page_slug];
	}

	return null;
}

/**
 * default result page contents
 */
function corona_test_results_create_default_pages( $pages = array() ) {
	global $corona_test_results_cfg;

	if ( empty($pages) ) {
		$pages = corona_test_results_get_result_pages_slugs();
	}

	foreach ( $pages as $page_slug ) {
		if ( !corona_test_results_get_page( $page_slug ) ) {
			$is_retrieval = $page_slug === 'result_retrieval';

			$suffix = $is_retrieval ? '' : ( ': ' . corona_test_results_get_page_state( $page_slug ) );

			$shortcode = $is_retrieval ? '[testresults_form]' : '[testresults_code formatted]';

			$post_details = array(
				'post_title'    => __('Test Result', 'corona-test-results') . $suffix,
				'post_content'  => "<!-- wp:shortcode -->\n$shortcode\n<!-- /wp:shortcode -->",
				'post_status'   => $is_retrieval ? 'publish' : 'draft',
				'post_type'		=> 'page'
			);

			if ( $is_retrieval ) {
				// translators: default results retrieval page URL slug
				$post_details['post_name'] = sanitize_title(__('testresult', 'corona-test-results'));
			}

			$new_page_id = wp_insert_post( $post_details );
			if ($new_page_id) {
				$corona_test_results_cfg['page_' . $page_slug] = $new_page_id;
			}
		}
	}
}

/*
 * activation hook (create DB and default pages)
 */
function corona_test_results_activation() {
    global $corona_test_results_cfg;

	require_once(__DIR__ . '/corona-test-results-admin.php');

	corona_test_results_conditionally_create_table();

	corona_test_results_create_default_pages();

	update_option( 'corona_test_results_options', $corona_test_results_cfg );

	$activationtime = get_option( 'corona_test_results_activationtime' );
	if (!$activationtime) {
		update_option( 'corona_test_results_activationtime', time() );
	}

}
register_activation_hook( corona_test_results__mainfile(), 'corona_test_results_activation');

/**
 * table, code generation and result related functions called from frontend and admin pages
 */
function corona_test_results_get_table_name() {
    global $wpdb;
    return $wpdb->prefix . "va_testresults";
}
function corona_test_results_get_code_chars() {
    $codeAlphabet = "ABCDEFGHJKLMNPQRSTUVWXYZ";
    $codeAlphabet.= "2345689";
    return $codeAlphabet;
}
function corona_test_results_get_code_regex() {
	return '/[' . corona_test_results_get_code_chars() . ']{' . CORONA_TEST_RESULTS_MIN_CODE_LENGTH . ',' . CORONA_TEST_RESULTS_CODE_COLUMN_LENGTH . '}/';
}
function corona_test_results_get_states( $as_slug = false ) {
    return array(
        0 => $as_slug ? 'pending' : __('Pending', 'corona-test-results'),
        1 => $as_slug ? 'positive' : __('Positive', 'corona-test-results'),
        2 => $as_slug ? 'negative' : __('Negative', 'corona-test-results')
    );
}

function vt_helper__is_flag( $flag, $atts ) {
	if ( empty($atts) ) return false;
    foreach ( $atts as $key => $value )
        if ( $value === $flag && is_int( $key ) ) return true;
    return false;
}

/**
 * wrappers for strings from default textdomain,
 * so translation tools like PoEdit ignore these
 */
function vt_helper__default_i18n__( $string ) {
    return __( $string );
}
function vt_helper__default_i18n_e( $string ) {
    return _e( $string );
}

/**
 * as we set the default options as early as possible,
 * the locale is not yet available at that time, so default options with i18n strings will always show in English.
 * This fixes the issue by re-translating those strings.
 */
function vt_helper__retranslate_option( $option_name ) {
	global $corona_test_results_cfg_defaults;
	$options = corona_test_results_get_options();

	if ( isset( $options[$option_name] ) ) {
		if ( $options[$option_name] === $corona_test_results_cfg_defaults[$option_name] ) {
			return __( $options[$option_name], 'corona-test-results' );
		} else {
			return $options[$option_name];
		}
	}
	return null;
}

/**
 * allow only styling HTML tags
 */
function vt_helper__esc_html( $input ) {
	return wp_kses( $input, array(
		'br' => array(),
		'em' => array(),
		'strong' => array(),
		'i' => array(),
		'b' => array(),
	) );
}

/**
 * replace placeholders in template
 */
function vt_helper__prep_tpl( $input ) {
	$options = corona_test_results_get_options();

	$output = $input;

	// hard-replace {{testdate}}
	$output = preg_replace( "/\{\{testdate\}\}/", date_i18n( get_option( 'date_format' ) ), $output );

	// if no custom field is used, but the custom field placeholder is present, remove it
	if ( empty($options['template_customfield']) ) {
		// if it would cause a line break, remove that as well
		$output = preg_replace_callback( "/(^|\r?\n)\s*\{\{customfield\}\}\s*(\r?\n|$)/", function( $matches ) {
			if ( preg_match( "/\n/", $matches[1] ) && preg_match( "/\n/", $matches[2] ) ) {
				return "\n";
			}

			return '';
		}, $output );
		// replace occurrences mid-string
		$output = preg_replace( "/\{\{customfield\}\}/", "", $output );
	}

	$output = preg_replace( "/\{\{([a-z_]+)\}\}/i", '<span data-vt-placeholder="$1"></span>', vt_helper__esc_html( $output ) );

	return $output;
}

/**
 * Returns the plugin information from the main file
 */
function corona_test_results_get_plugin_info() {
	return get_plugin_data( corona_test_results__mainfile(), false );
}

/**
 * Returns a link to the plugin author
 */
function corona_test_results_get_author_link() {
	$pluginData = corona_test_results_get_plugin_info();
	return "<a href='" . esc_attr( esc_url( $pluginData['AuthorURI'] ) ) . "' target='_blank'>" . $pluginData['Author'] . "</a>";
}

/**
 * returns a link to the shop URL for the premium version
 */
function corona_test_results_premium_shop_url() {
	return 'https://shop.48design.com/produkt/wordpress/corona-testergebnis-plugin-premium/';
}

